@@ -440,6 +440,43 @@ Initially, this is a handle to the active console screen buffer, CONOUT$.")
               (lpReserved c-pointer))
   (:return-type boolean))
 
+;; http://msdn.microsoft.com/library/en-us/winui/winui/windowsuserinterface/dataexchange/clipboard/clipboardreference/clipboardfunctions/getclipboarddata.asp
+(def-call-out OpenClipboard (:library user32)
+  (:arguments (hWndNewOwner handle)) (:return-type boolean))
+(def-call-out CloseClipboard (:library user32)
+  (:arguments) (:return-type boolean))
+(def-call-out EmptyClipboard (:library user32)
+  (:arguments) (:return-type boolean))
+(def-call-out EnumClipboardFormats (:library user32)
+  (:arguments (format uint)) (:return-type uint))
+(def-call-out GetClipboardData (:library user32)
+  (:arguments (format uint)) (:return-type handle))
+(def-call-out GetClipboardFormatNameA (:library user32)
+  (:arguments (format uint)
+              (buffer (c-ptr (c-array-max character #.BUFSIZ)) :out :alloca)
+              (size int)) ; always pass BUFSIZ as the second argument
+  (:return-type int))
+(def-call-out GetClipboardOwner (:library user32)
+  (:arguments) (:return-type handle))
+(def-call-out GetClipboardSequenceNumber (:library user32)
+  (:arguments) (:return-type dword))
+(def-call-out GetClipboardViewer (:library user32)
+  (:arguments) (:return-type handle))
+(def-call-out GetOpenClipboardWindow (:library user32)
+  (:arguments) (:return-type handle))
+(def-call-out GetPriorityClipboardFormat (:library user32)
+  (:arguments (formats (c-array-max uint #.BUFSIZ)) (size int))
+  (:return-type int))
+(def-call-out IsClipboardFormatAvailable (:library user32)
+  (:arguments (format uint)) (:return-type boolean))
+(def-call-out RegisterClipboardFormat (:library user32)
+  (:arguments (format c-string)) (:return-type uint))
+(def-call-out RemoveClipboardFormatListener (:library user32)
+  (:arguments (hwnd handle)) (:return-type boolean))
+(def-call-out SetClipboardData (:library user32)
+  (:arguments (format uint) (data handle)) (:return-type handle))
+(def-call-out SetClipboardViewer (:library user32)
+  (:arguments (hwnd handle)) (:return-type handle))
 
 ;; ==========================================================================
 
