@@ -16,7 +16,7 @@
 ; along with this program; if not, write to the Free Software Foundation,
 ; Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ;
-; $Id: oracle.lisp,v 1.23 2005/08/26 13:47:23 hin Exp $
+; $Id: oracle.lisp,v 1.26 2006/08/01 10:01:15 hin Exp $
 
 (defpackage "ORACLE"
   (:documentation
@@ -31,6 +31,8 @@
 (in-package "ORACLE")
 (pushnew :oracle *features*)
 
+(setf (documentation (find-package "ORACLE") 'sys::impnotes) "oracle")
+
 ; Use "C" as foreign language
 (default-foreign-language :stdc)
 
@@ -402,7 +404,7 @@ Arguments: none
 
 ; FETCH-ALL
 (defun fetch-all (&optional max-rows (result-type 'ARRAY) (item-type 'ARRAY))
-"(ORACLE:FETCH-ALL(&optional max-rows (result-type 'ARRAY) (item-type 'ARRAY))
+  "(ORACLE:FETCH-ALL(&optional max-rows (result-type 'ARRAY) (item-type 'ARRAY))
 
 Fetch all rows from a query and return result as a sequence of
 sequences.  Arguments (all optional) are:
@@ -417,14 +419,25 @@ sequences.  Arguments (all optional) are:
   (when (not (find item-type '(array list)))
 	(db-error (cat "Item type '" item-type "' should be 'ARRAY or 'LIST")))
 
-  (do ((result (make-array 100 :element-type item-type
-			   :fill-pointer 0 :adjustable t))
-       (count 0 (1+ count))
-       (row (oracle:fetch) (oracle:fetch)))
-      ((or (null row) (when max-rows (>= count max-rows)))
-       (coerce result result-type))
-      (vector-push-extend (coerce row item-type)
-				      result)))
+  (cond ((and max-rows (< max-rows 0))
+		 (db-error (cat "Negative count [" max-rows "] given")))
+		((and max-rows (= 0 max-rows))
+		 ;; Do nothing
+		 (coerce nil result-type))
+		(t
+		 ;; Want at least one row
+		 (do ((result (make-array 100 :element-type item-type :fill-pointer 0 :adjustable t))
+			  (count 1 (1+ count))
+			  (row nil)
+			  (at-eof nil))
+			 
+			 (at-eof (coerce result result-type))
+		   
+		   (setf row (oracle:fetch))
+		   (when row
+			 (vector-push-extend (coerce row item-type) result))
+		   (when (or (not row) (and max-rows (>= count max-rows)))
+			 (setf at-eof t))))))
 
 ;----------------------------------------------
 
@@ -744,7 +757,6 @@ Argument: none
 (defun row-to-result (row result-type)
   (cond ((null row) nil)
         ((eq result-type 'ARRAY) row)
-        ; ((eq result-type 'HASH) (pairs-to-hash (row-to-result row 'PAIRS)))
         ((eq result-type 'HASH) (array-to-hash (row-to-result row 'ARRAY)))
         ((eq result-type 'PAIRS)
          (let ((colinfo (oracle_column_info (curconn))))
@@ -757,7 +769,6 @@ Argument: none
                     colinfo row)))))
         (t (db-error (cat "Invalid result type '" result-type "' given - should be 'ARRAY, 'PAIRS or 'HASH")))))
 
-
 ; CHECK-SUCCESS
 ; Check Oracle success code after calling a function.  Assumes (check-connection) was called!
 (defun check-success ()
@@ -782,7 +793,10 @@ Argument: none
 				   (setf *read-default-float-format* 'double-float)
 				   (read-from-string val))
 			   (setf *read-default-float-format* old-default-format))))
-          ((find dtype '("VARCHAR" "DATE" "CHAR" "VARCHAR2" "LONG" "RAW" "LONG RAW" "CLOB" "ROWID DESC") :test #'equal)
+          ((find dtype '("VARCHAR" "DATE" "CHAR" "VARCHAR2" "LONG" "RAW" "LONG RAW" "CLOB" "ROWID DESC"
+                         "ANSI DATE" "TIME" "TIME WITH TIME ZONE" "TIMESTAMP" "TIMESTAMP WITH TIME ZONE"
+                         "INTERVAL YEAR TO MONTH" "INTERVAL DAY TO SECOND" "TIMESTAMP WITH LOCAL TZ"
+                         ) :test #'equal)
            val)
           (t (db-error (cat "Unsupported data type '" dtype "'"))))))
 
