@@ -598,45 +598,47 @@ Supplies some HTTP/1.0 headers and calls `with-html-output'."
     (values socket id com keep-alive)))
 
 (clos:defmethod inspect-frontend ((insp inspection) (frontend (eql :http)))
-  (do ((server
-        (let* ((server (socket-server)) (port (socket-server-port server))
-               (host (machine-instance)))
-          (when (> *inspect-debug* 0)
-            (format t "~&~s [~s]: server: ~s~%"
-                    'inspect-frontend frontend server))
-          (browse-url (format nil "http://~a:~d/0/:s"
-                              (if *inspect-browser* "127.0.0.1"
-                                  (subseq host 0 (position #\Space host)))
-                              port)
-                      :browser *inspect-browser*)
-          server))
-       sock id com keep-alive)
-      ((eq com :q) (socket-server-close server)
-       (when (open-stream-p sock)
-         (do () ((null (read-char-no-hang sock))))
-         (close sock)))
-    (setf (values sock id com keep-alive) (http-command server :socket sock))
-    (when id
-      (if (eq com :q)
-          (with-http-output (out sock :keep-alive keep-alive
-                                 :debug *inspect-debug*
-                                 :title "inspect" :footer nil)
-            (with-tag (:h1) (princ "thanks for using inspect" out))
-            (with-tag (:p) (princ "you may close this window now" out)))
-          (if (setq insp (get-insp id com))
-              (print-inspection insp sock frontend :keep-alive keep-alive)
-              (with-http-output (out sock :keep-alive keep-alive
-                                     :debug *inspect-debug*
-                                     :title "inspect" :footer nil)
-                (with-tag (:h1)
-                  (format out "error: wrong command: ~:d/~s" id com))
-                (with-tag (:p)
-                  (princ "either this is an old inspect session, or a " out)
-                  (with-tag (:a :href "https://sourceforge.net/bugs/?func=addbug&group_id=1802") (princ "bug" out))))))
-      (when (> *inspect-debug* 0)
-        (format t "~s [~s]: cmd:~d/~s id:~d~%" 'inspect-frontend frontend
-                id com (insp-id insp))))))
-
+  (let ((server
+         (let* ((server (socket-server)) (port (socket-server-port server))
+                (host (machine-instance)))
+           (when (> *inspect-debug* 0)
+             (format t "~&~s [~s]: server: ~s~%"
+                     'inspect-frontend frontend server))
+           (browse-url (format nil "http://~a:~d/0/:s"
+                               (if *inspect-browser* "127.0.0.1"
+                                   (subseq host 0 (position #\Space host)))
+                               port)
+                       :browser *inspect-browser*)
+           server))
+        sock id com keep-alive)
+    (unwind-protect
+         (loop (when (eq com :q) (return))
+           (setf (values sock id com keep-alive)
+                 (http-command server :socket sock))
+           (when id
+             (if (eq com :q)
+               (with-http-output (out sock :keep-alive keep-alive
+                                      :debug *inspect-debug*
+                                      :title "inspect" :footer nil)
+                 (with-tag (:h1) (princ "thanks for using inspect" out))
+                 (with-tag (:p) (princ "you may close this window now" out)))
+               (if (setq insp (get-insp id com))
+                 (print-inspection insp sock frontend :keep-alive keep-alive)
+                 (with-http-output (out sock :keep-alive keep-alive
+                                        :debug *inspect-debug*
+                                        :title "inspect" :footer nil)
+                   (with-tag (:h1)
+                     (format out "error: wrong command: ~:d/~s" id com))
+                   (with-tag (:p)
+                     (princ "either this is an old inspect session, or a " out)
+                     (with-tag (:a :href "https://sourceforge.net/bugs/?func=addbug&group_id=1802") (princ "bug" out))))))
+             (when (> *inspect-debug* 0)
+               (format t "~s [~s]: cmd:~d/~s id:~d~%" 'inspect-frontend
+                       frontend id com (insp-id insp)))))
+      (socket-server-close server)
+      (when (open-stream-p sock)
+        (do () ((null (read-char-no-hang sock))))
+        (close sock)))))
 ;;;
 ;;; the juice
 ;;;
